@@ -134,10 +134,18 @@ static void sn_exec(char* name, char* bin_name, char *desc);
 #define UP   0
 #define DOWN 1
 
-#define KEY_JOG_DOWN	65364
-#define KEY_JOG_UP	65362
-#define KEY_OK		65293
-#define KEY_CANCEL	65307
+#define KEY_CANCEL		65307
+#define KEY_OK			65293
+#define KEY_JOG_RIGHT		65361
+#define KEY_JOG_UP		65362
+#define KEY_JOG_LEFT		65363
+#define KEY_JOG_DOWN		65364
+#define KEY_CERH1_VOL_DOWN	269025041
+#define KEY_CERH1_VOL_UP	269025043
+#define KEY_CERH1_PLAY		269025044
+#define KEY_CERH1_STOP		269025045
+#define KEY_CERH1_PREV		269025046
+#define KEY_CERH1_NEXT		269025047
 
 #define MBMAX(x,y) ((x>y)?(x):(y))
 
@@ -684,39 +692,57 @@ static gboolean key_event_callback( GtkWidget      *widget,
 				    GdkEventKey	   *event,
 				    gpointer        data )
 {
-  /* Return if xmms control is disabled */
-  if (!XmmsControl)
+  /* Return if xmms isn't running */
+  if (WEXITSTATUS(system("pidof -s xmms > /dev/null")) > 0) {
     return FALSE;
+  }
 
-  if (event->type == GDK_KEY_PRESS) {
-    //printf("key pressed (%d)\n", event->keyval);
-  } else if (event->type == GDK_KEY_RELEASE) {
-    //printf("key released (%d)\n", event->keyval);
-
-    /* Return if xmms isn't running */
-    if (WEXITSTATUS(system("pidof -s xmms > /dev/null")) > 0) {
-      //printf("Could not execute pidof or xmms not running\n");
-      return FALSE;
-    }
-
-    switch (event->keyval)
+  /* Return if xmms control is disabled but still allow control via CE-RH1 */
+  if (!XmmsControl) {
+    switch(event->keyval)
       {
       case KEY_JOG_DOWN:
-	//printf("xmms --fwd\n");
+      case KEY_JOG_UP:
+      case KEY_JOG_LEFT:
+      case KEY_JOG_RIGHT:
+      case KEY_OK:
+      case KEY_CANCEL:
+	return FALSE;
+      }
+  }
+
+  if (event->type == GDK_KEY_PRESS) {
+    // Key pressed
+  } else if (event->type == GDK_KEY_RELEASE) {
+    // Key released
+    switch (event->keyval)
+      {
+      case KEY_CERH1_NEXT:
+      case KEY_JOG_DOWN:
+      case KEY_JOG_LEFT:
 	fork_exec("xmms --fwd");
 	break;
+      case KEY_CERH1_PREV:
       case KEY_JOG_UP:
-	//printf("xmms --rew\n");
+      case KEY_JOG_RIGHT:
 	fork_exec("xmms --rew");
 	break;
+      case KEY_CERH1_PLAY:
       case KEY_OK:
-	//printf("xmms --play-pause\n");
 	fork_exec("xmms --play-pause");
 	break;
+      case KEY_CERH1_STOP:
       case KEY_CANCEL:
-	//printf("xmms --stop\n");
 	fork_exec("xmms --stop");
 	break;
+      case KEY_CERH1_VOL_UP:
+	fork_exec("xmms --vol-up");
+	break;
+      case KEY_CERH1_VOL_DOWN:
+	fork_exec("xmms --vol-down");
+	break;
+	/* default:
+	   printf("unknown key: %d\n", event->keyval); */
       }
   } else {
     //printf("unknown event\n");
